<?php
/**
 * Holler Box Pro Functions
 * @since       0.1.0
 */


// Exit if accessed directly
if( !defined( 'ABSPATH' ) ) exit;

if( !class_exists( 'Holler_Pro_Functions' ) ) {

    /**
     * Holler_Pro_Functions class
     *
     * @since       0.2.0
     */
    class Holler_Pro_Functions {

        /**
         * @var         Holler_Pro_Functions $instance The one true Holler_Pro_Functions
         * @since       0.2.0
         */
        private static $instance;
        public static $errorpath = '../php-error-log.php';
        public static $active = array();
        // sample: error_log("meta: " . $meta . "\r\n",3,self::$errorpath);

        /**
         * Get active instance
         *
         * @access      public
         * @since       0.2.0
         * @return      object self::$instance The one true Holler_Pro_Functions
         */
        public static function instance() {
            if( !self::$instance ) {
                self::$instance = new Holler_Pro_Functions();
                self::$instance->hooks();
            }

            return self::$instance;
        }


        /**
         * Include necessary files
         *
         * @access      private
         * @since       0.2.0
         * @return      void
         */
        private function hooks() {

            add_action( 'wp_enqueue_scripts', array( $this, 'scripts_styles' ) );
            add_filter( 'hwp_display_notification', array( $this, 'display_filters' ), 10, 3 );
            add_filter( 'hollerbox_content', array( $this, 'maybe_do_popout' ), 10, 2 );

            add_shortcode( 'holler-upgrade', array( $this, 'holler_upgrade_shortcode' ) );

        }

        /**
         * Load scripts
         *
         * @since       0.1.0
         * @return      void
         */
        public function scripts_styles( $hook ) {

            // Use minified libraries if SCRIPT_DEBUG is turned off
            $suffix = ( defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ) ? '' : '.min';

            wp_enqueue_script( 'holler-pro-js', Holler_Box_Pro_URL . 'assets/js/holler-pro' . $suffix . '.js', array( 'holler-js' ), Holler_Box_Pro_VER, true );
            wp_enqueue_style( 'holler-pro-css', Holler_Box_Pro_URL . 'assets/css/holler-pro-frontend' . $suffix . '.css', array('holler-css'), Holler_Box_Pro_VER );

        }

        /**
         * Adds more conditions for when we should show/hide the item
         * $box_id is holler box cpt ID, $post_id is current global post->ID
         *
         * @since       0.1.0
         * @return      void
         */
        public function display_filters( $show_it, $box_id, $post_id ) {

            $hbf_class = new Holler_Functions();

            $show_on = get_post_meta( $box_id, 'show_on', 1 );
            $show_on_tags = get_post_meta( $box_id, 'hwp_show_on_tags', 1 );
            $show_on_cats = get_post_meta( $box_id, 'hwp_show_on_cats', 1 );
            $show_on_ids = get_post_meta( $box_id, 'hwp_show_on_ids', 1 );
            $show_exclude_pages = get_post_meta( $box_id, 'hwp_show_exclude_pages', 1 );
            $show_on_types = get_post_meta( $box_id, 'hwp_show_on_types', 1 );

            // only run these checks if settings is "certain pages"
            if( $show_on != 'limited' )
                return $show_it; 

            // tags conditionals
            if( !empty( $show_on_tags ) ) {

                // explode tags and check current tag
                $posttags = wp_get_post_terms( $post_id, 'post_tag', array("fields" => "names") );

                // clean up and return array
                $arr = self::clean_string_to_arr( $show_on_tags );

                // is this post in the show_on_tags tag?
                $tag_intersect = array_intersect( $posttags, $arr );

                // if we have a match, show it
                if( !empty( $tag_intersect ) )
                    $show_it = true;

            }

            // category conditionals
            if( !empty( $show_on_cats ) ) {

                // explode cats and check current cat
                $postcats = wp_get_post_terms( $post_id, 'category', array("fields" => "names") );

                // clean up and return array
                $arr = self::clean_string_to_arr( $show_on_cats );

                // is this post in the show_on_cats category?
                $cat_intersect = array_intersect( $postcats, $arr );

                // if we have a match, show it
                if( !empty( $cat_intersect ) )
                    $show_it = true;

            }

            // post type conditionals
            if( !empty( $show_on_types ) ) {

                $type = get_post_type( $post_id );

                // clean up and return array
                $arr = self::clean_string_to_arr( $show_on_types );

                // check if this post type is in the show_on_types array
                if( in_array( $type, $arr ) )
                    $show_it = true;

            }

            // Don't show if page is in exclude ids
            if( !empty( $show_exclude_pages ) ) {

                // turn into array of ids
                $arr = $hbf_class::titles_to_ids( $show_exclude_pages );

                //wp_die( serialize( $arr));

                if( in_array( $post_id, $arr ) )
                    $show_it = false;

            } 

            return $show_it;

        }

        /*
         * Strip whitespace and return array
         */
        public static function clean_string_to_arr( $string ) {

            // remove spaces
            $string = preg_replace('/\s+/', '', $string );

            // explode into array
            $arr = explode( ",", $string );

            return $arr;

        }

        /*
         * Maybe add popout
         */
        public function maybe_do_popout( $content, $id ) {

            if( get_post_meta( $id, 'hwp_type', 1) != 'popout' )
                return $content;

            $popout_content = get_post_meta( $id, 'hwp_popout_editor', 1);
            $btn_text = get_post_meta( $id, 'hwp_popout_btn_text', 1);
            $text = ( $btn_text ? $btn_text : 'Click here' );
            if( !$popout_content )
                return $content;

            $content .= '<a href="#" class="hwp-button hwp-expand-side">' . $text . '</a><div class="hwp-show-expanded">';
            $content .= $popout_content;
            $content .= '</div>';

            return $content;
        }

        /*
         * Shortcode for content upgrades
         */
        public function holler_upgrade_shortcode( $atts, $content = null ) {

            $a = shortcode_atts( array(
                'link' => 'Click here.',
                'id' => '',
            ), $atts );

            return '<p class="holler-upgrade">' . $content . ' <a class="holler-show" data-id="' . $a['id'] . '" href="#">' . $a['link'] . '</a></p>';

        }

    }

    $holler_pro_functions = new Holler_Pro_Functions();
    $holler_pro_functions->instance();

} // end class_exists check